#!/bin/bash

# 
# eCross Toolchain Builder
#                  
# Copyright (C) 2010 by Ole Reinhardt <ole.reinhardt@embedded-it.de>, Thermotemp GmbH
# 
# This script is based on the YAGARTO toolchain build scripts
# Copyright (C) 2006-2010 by Michael Fischer <Michael.Fischer@yagarto.de>
#                                                                            
# This program is free software; you can redistribute it and/or modify       
# it under the terms of the GNU General Public License as published by       
# the Free Software Foundation; either version 2 of the License, or          
# (at your option) any later version.                                        
#                                                                            
# This program is distributed in the hope that it will be useful,            
# but WITHOUT ANY WARRANTY; without even the implied warranty of             
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              
# GNU General Public License for more details.                               
#                                                                            
# You should have received a copy of the GNU General Public License          
# along with this program; if not, write to the Free Software                
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA 
# 

# Please place the following files to ./download
#	- binutils-2.21.tar.bz2
#	- expat-2.0.1.tar.gz
#	- gcc-core-4.5.2.tar.bz2
#	- gcc-g++-4.5.2.tar.bz2
#	- gdb-7.2.tar.bz2
#	- gmp-4.3.2.tar.bz2
#	- mpc-0.8.1.tar.gz
#	- mpfr-2.4.2.tar.bz2
#	- newlib-1.19.0.tar.gz
#
# Place the patches into ./patches directory:
#
#	- binutils-2.21.patch  
#	- gcc-4.5.2.patch  

export TARGET=arm-eCross-eabi
export PREFIX=$(pwd)/$TARGET
export TOOLS_AND_LIBS_DIR=$(pwd)/tools_and_libs
export MAKE=make

mkdir -p $PREFIX
mkdir -p $TOOLS_AND_LIBS_DIR

export PATH="$PATH:$PREFIX/bin"

#---------------------------------------------------------------------------------
# Version specification
#---------------------------------------------------------------------------------

export BINUTILS_VER=2.21
export GCC_VER=4.5.2
export NEWLIB_VER=1.19.0
export GDB_VER=7.2
export GMP_VER=4.3.2
export MPFR_VER=2.4.2
export MPC_VER=0.8.1
export EXPAT_VER=2.0.1

#---------------------------------------------------------------------------------
# Source specification
#---------------------------------------------------------------------------------

export BINUTILS_SRC="binutils-$BINUTILS_VER"
export GCC_SRC="gcc-$GCC_VER"
export NEWLIB_SRC="newlib-$NEWLIB_VER"
export GDB_SRC="gdb-$GDB_VER"
export GMP_SRC="gmp-$GMP_VER"
export MPFR_SRC="mpfr-$MPFR_VER"
export MPC_SRC="mpc-$MPC_VER"
export EXPAT_SRC="expat-$EXPAT_VER"

#---------------------------------------------------------------------------------
# Stage 0: Extract sources and apply patches
#---------------------------------------------------------------------------------
function stage0 ()
{

	mkdir src
	cd src

	for f in ../download/*.tar.gz
	do 
		echo Extracting $f
		tar -xzf $f
	done

	for f in ../download/*.tar.bz2
	do
		echo Extracting $f
		tar -xjf $f
	done

	cd ..

	echo
	echo "Patching $GCC_SRC"
	patch -p1 -d src/$GCC_SRC < patches/$GCC_SRC.patch || { echo "Error patching "$GCC_SRC; exit; }
	echo

#	echo
#	echo "Patching $NEWLIB_SRC"
#	patch -p1 -d src/$NEWLIB_SRC < patches/$NEWLIB_SRC.patch || { echo "Error patching "$NEWLIB_SRC; exit; }
#	patch -p1 -d src/$NEWLIB_SRC < patches/$NEWLIB_SRC-include-sys-types.h.patch || { echo "Error patching "$NEWLIB_SRC; exit; }
#	echo

	echo
	echo "Patching $BINUTILS_SRC"
	patch -p1 -d src/$BINUTILS_SRC < patches/$BINUTILS_SRC.patch || { echo "Error patching "$BINUTILS_SRC; exit; }
	echo


}


#---------------------------------------------------------------------------------
# Stage 1: Build expat
#---------------------------------------------------------------------------------

function stage1 ()
{

	echo ----- Building expat ------
	sleep 5

#	export CFLAGS=-D__USE_MINGW_ACCESS

	mkdir -p expat-build
	cd expat-build

	../src/$EXPAT_SRC/configure --prefix=$TOOLS_AND_LIBS_DIR --disable-shared \
		|| { echo "Error configuring expat"; exit 1; }

	$MAKE || { echo "Error building expat"; exit 1; }
	$MAKE install || { echo "Error installing expat"; exit 1; }

	cd ..
	echo ----- End of expat build ------
}

#---------------------------------------------------------------------------------
# Stage 2: Build gmp
#---------------------------------------------------------------------------------

function stage2 ()
{

	echo ----- Building gmp ------
	sleep 5

#	export CFLAGS=-D__USE_MINGW_ACCESS

	mkdir -p gmp-build
	cd gmp-build

	../src/$GMP_SRC/configure ABI="32" \
		--prefix=$TOOLS_AND_LIBS_DIR \
		|| { echo "Error configuring gmp"; exit 1; }

	$MAKE || { echo "Error building gmp"; exit 1; }
	$MAKE check || { echo "Error checking gmp"; exit 1; }
	$MAKE install || { echo "Error installing gmp"; exit 1; }

	rm $TOOLS_AND_LIBS_DIR/lib/*.dylib

#	cp ../src/$GMP_SRC/gmp-impl.h .
#	cp ../src/$GMP_SRC/longlong.h .

	cd ..
	echo ----- End of gmp build ------
}

#---------------------------------------------------------------------------------
# Stage 3: Build mpfr
#---------------------------------------------------------------------------------

function stage3 ()
{

	echo ----- Building mpfr ------
	sleep 5

#	export CFLAGS=-D__USE_MINGW_ACCESS

	mkdir -p mpfr-build
	cd mpfr-build

	../src/$MPFR_SRC/configure ABI="32" \
		--prefix=$TOOLS_AND_LIBS_DIR --disable-shared \
		--with-gmp-build=../gmp-build \
		|| { echo "Error configuring mpfr"; exit 1; }

	$MAKE || { echo "Error building mpfr"; exit 1; }
	$MAKE install || { echo "Error installing mpfr"; exit 1; }

	cd ..
	echo ----- End of mpfr build ------
}


#---------------------------------------------------------------------------------
# Stage 4: Build mpc
#---------------------------------------------------------------------------------

function stage4 ()
{

	echo ----- Building mpc ------
	sleep 5

#	export CFLAGS=-D__USE_MINGW_ACCESS

	mkdir -p mpc-build
	cd mpc-build

	../src/$MPC_SRC/configure ABI="32" \
	   CPPFLAGS=-I$TOOLS_AND_LIBS_DIR/include LDFLAGS=-L$TOOLS_AND_LIBS_DIR/lib \
		--prefix=$TOOLS_AND_LIBS_DIR --enable-static --disable-shared \
		|| { echo "Error configuring mpc"; exit 1; }

	$MAKE || { echo "Error building mpc"; exit 1; }
	$MAKE install || { echo "Error installing mpc"; exit 1; }

	cd ..
	echo ----- End of mpc build ------
}


#---------------------------------------------------------------------------------
# Stage 5: Build binutils
#---------------------------------------------------------------------------------

function stage5 ()
{

	echo ----- Building binutils ------
	sleep 5

#	export CFLAGS=-D__USE_MINGW_ACCESS

	mkdir -p binutils-build
	cd binutils-build

	../src/$BINUTILS_SRC/configure \
		--target=$TARGET --prefix=$PREFIX \
		--enable-interwork --enable-multilib \
		--disable-nls --disable-shared --disable-threads \
		--with-gcc --with-gnu-as --with-gnu-ld --enable-gold \
		|| { echo "Error configuring binutils"; exit 1; }
#		--with-gcc --with-gnu-as --with-gnu-ld --enable-gold \

	$MAKE || { echo "Error building binutils"; exit 1; }
	$MAKE install || { echo "Error installing binutils"; exit 1; }

	cd ..
	echo ----- End of binutils build ------
}

#---------------------------------------------------------------------------------
# Stage 6: Build bootstrap gcc
#---------------------------------------------------------------------------------

function stage6 ()
{

	echo ----- Building bootstrap gcc ------
	sleep 5

#	export CFLAGS=-D__USE_MINGW_ACCESS

	mkdir -p gcc-build
	cd gcc-build

	CFLAGS="-I$TOOLS_AND_LIBS_DIR/include" \
	LDFLAGS="-L$TOOLS_AND_LIBS_DIR/lib" \
	../src/$GCC_SRC/configure \
		--target=$TARGET --prefix=$PREFIX \
		--disable-nls --disable-shared --disable-threads \
		--with-gcc --with-gnu-ld --with-gnu-as --with-dwarf2 \
		--enable-languages=c,c++ --enable-interwork --enable-multilib \
		--with-newlib --with-headers=../src/newlib-$NEWLIB_VER/newlib/libc/include \
		--disable-libssp --disable-libstdcxx-pch --disable-libmudflap \
		--disable-libgomp --with-system-zlib --enable-lto -v \
		|| { echo "Error configuring gcc"; exit 1; }
#		--disable-libgomp --enable-lto -v \

	mkdir -p libiberty libcpp fixincludes

	$MAKE all-gcc || { echo "Error building bootstrap gcc"; exit 1; }
	$MAKE install-gcc || { echo "Error installing  bootstrap gcc"; exit 1; }

	cd ..
	echo ----- End of bootstrap gcc build ------
}

#---------------------------------------------------------------------------------
# Stage 7: Build newlib
#---------------------------------------------------------------------------------

function stage7 ()
{

	echo ----- Building newlib ------
	sleep 5

	mkdir -p newlib-build
	cd newlib-build
	mkdir -p etc

	../src/$NEWLIB_SRC/configure \
		--target=$TARGET --prefix=$PREFIX \
		--enable-interwork --enable-multilib \
		--disable-newlib-supplied-syscalls \
		|| { echo "Error configuring newlib"; exit 1; }

	$MAKE || { echo "Error building newlib"; exit 1; }
	$MAKE install || { echo "Error installing newlib"; exit 1; }

	cd ..
	echo ----- End of newlib build ------
}

#---------------------------------------------------------------------------------
# Stage 8: Build gcc
#---------------------------------------------------------------------------------

function stage8 ()
{

	echo ----- Building gcc ------
	sleep 5

	cd gcc-build

	$MAKE || { echo "Error building gcc (second stage)"; exit 1; }
	$MAKE install || { echo "Error installing gcc (second stage)"; exit 1; }

	cd ..
}

#---------------------------------------------------------------------------------
# Stage 9: Build gdb
#---------------------------------------------------------------------------------

function stage9 ()
{

	echo ----- Building gdb ------
	sleep 5

	mkdir -p gdb-build
	cd gdb-build

	../src/$GDB_SRC/configure --target=$TARGET --prefix=$PREFIX --disable-nls \
		--with-libexpat-prefix=$TOOLS_AND_LIBS_DIR \
		|| { echo "Error configuring gdb"; exit 1; }

	$MAKE || { echo "Error building gdb"; exit 1; }

	$MAKE install || { echo "Error installing gdb"; exit 1; }

	cd ..
}

#---------------------------------------------------------------------------------
# Stage 10: strip binaries
#---------------------------------------------------------------------------------

function stage10 ()
{

	echo ----- Stripping binaries ------
	sleep 5

	for f in \
		$PREFIX/bin/* \
		$PREFIX/$TARGET/bin/* \
		$PREFIX/libexec/gcc/$TARGET/$GCC_VER/*
	do
		strip $f
	done
}

if [ -z "$1" ] ; then stage=0 ; else stage=$1 ; fi

if [ $stage -lt 1 ] ; then stage0 ; fi
if [ $stage -lt 2 ] ; then stage1 ; fi
if [ $stage -lt 3 ] ; then stage2 ; fi
if [ $stage -lt 4 ] ; then stage3 ; fi
if [ $stage -lt 5 ] ; then stage4 ; fi
if [ $stage -lt 6 ] ; then stage5 ; fi
if [ $stage -lt 7 ] ; then stage6 ; fi
if [ $stage -lt 8 ] ; then stage7 ; fi
if [ $stage -lt 9 ] ; then stage8 ; fi
if [ $stage -lt 10 ] ; then stage9 ; fi
if [ $stage -lt 11 ] ; then stage10 ; fi

